@@ -28,9 +28,6 @@ REST_RECORDING_DETAILS = "pvr/details/{0}"
 REST_PATH_INFO = "system/information"
 REST_PATH_DEVICEINFO = "system/deviceinformation"
 
-# Generic Constants
-DEFAULT_ENCODING = "utf-8"
-
 # Sky specific constants
 CURRENT_URI = "CurrentURI"
 CURRENT_TRANSPORT_STATE = "CurrentTransportState"
@@ -99,6 +96,12 @@ COMMANDS = {
 
 RESPONSE_OK = 200
 
+KNOWN_COUNTRIES = {
+    "GBR": "GBR",
+    "IRL": "GBR",
+    "ITA": "ITA",
+}
+
 TEST_CHANNEL_LIST = {
     "documentId": "2416",
     "services": [
@@ -141,6 +144,19 @@ TEST_CHANNEL_LIST = {
             "t": "Sky Arte HD",
             "xsg": 1,
         },
+        {
+            "c": "999",
+            "dvbtriplet": "64511.6400.11074",
+            "schedule": True,
+            "servicetype": "DSAT",
+            "servicetypes": ["DSAT"],
+            "sf": "hd",
+            "sg": 21,
+            "sid": "99",
+            "sk": 99,
+            "t": "Fake Channel",
+            "xsg": 1,
+        },
         {
             "c": "101",
             "dvbtriplet": "2.2048.10353",
