@@ -11,7 +11,7 @@ import pycountry
 from datetime import datetime, timedelta
 from http import HTTPStatus
 
-from ws4py.client.threadedclient import WebSocketClient
+import websocket
 
 from .const import (
     SKY_PLAY_URN,
@@ -31,7 +31,6 @@ from .const import (
     REST_RECORDING_DETAILS,
     REST_PATH_INFO,
     REST_PATH_DEVICEINFO,
-    DEFAULT_ENCODING,
     CURRENT_URI,
     CURRENT_TRANSPORT_STATE,
     APP_STATUS_VISIBLE,
@@ -47,6 +46,7 @@ from .const import (
     SKY_STATE_ON,
     SKY_STATE_OFF,
     APP_EPG,
+    KNOWN_COUNTRIES,
 )
 from .const import TEST_CHANNEL_LIST
 from .channel import Channel
@@ -65,21 +65,20 @@ class SkyQRemote:
         self, host, overrideCountry=None, test_channel=None, port=49160, jsonport=9006,
     ):
         """Stand up a new SkyQ box."""
-        self._host = host
+        self.host = host
+        self.country = None
+        self.serialNumber = None
+        self.deviceSetup = False
+        self._channel = None
         self._test_channel = test_channel
         self._port = port
         self._jsonport = jsonport
         self._overrideCountry = overrideCountry
-        self.deviceSetup = False
-        self.channel = None
         self._soapControlURL = None
         self._lastEpg = None
+        self._currentApp = APP_EPG
 
         self.powerStatus()
-        if not self.deviceSetup:
-            _LOGGER.error(
-                f"E0060 - Device not switched on during restart: {self._host}"
-            )
 
     def powerStatus(self) -> str:
         """Get the power status of the Sky Q box."""
@@ -100,11 +99,11 @@ class SkyQRemote:
             return SKY_STATE_STANDBY
         except (requests.exceptions.ConnectionError):
             _LOGGER.info(
-                f"I0010 - Device has control URL but connection request failed: {self._host}"
+                f"I0010 - Device has control URL but connection request failed: {self.host}"
             )
             return SKY_STATE_OFF
         except Exception as err:
-            _LOGGER.exception(f"X0060 - Error occurred: {self._host} : {err}")
+            _LOGGER.exception(f"X0060 - Error occurred: {self.host} : {err}")
             return SKY_STATE_STANDBY
 
     def getCurrentState(self):
@@ -125,28 +124,26 @@ class SkyQRemote:
     def getActiveApplication(self):
         """Get the active application on Sky Q box."""
         try:
-            result = APP_EPG
             apps = self._callSkyWebSocket(WS_CURRENT_APPS)
             if apps is None:
-                return result
-            app = next(a for a in apps["apps"] if a["status"] == APP_STATUS_VISIBLE)[
-                "appId"
-            ]
+                return self._currentApp
 
-            result = app
+            self._currentApp = next(
+                a for a in apps["apps"] if a["status"] == APP_STATUS_VISIBLE
+            )["appId"]
 
-            return result
+            return self._currentApp
         except Exception:
-            return result
+            return self._currentApp
 
     def getCurrentMediaJSON(self):
         """Get the currently playing media on the SkyQ box as json."""
         media = self.getCurrentMedia()
-        if media:
-            return media.as_json()
-        else:
+        if not media:
             return None
 
+        return media.as_json()
+
     def getCurrentMedia(self):
         """Get the currently playing media on the SkyQ box."""
         channel = None
@@ -180,11 +177,11 @@ class SkyQRemote:
     def getEpgDataJSON(self, sid, epgDate):
         """Get EPG data for the specified channel as json."""
         channel = self.getEpgData(sid, epgDate)
-        if channel:
-            return channel.as_json()
-        else:
+        if not channel:
             return None
 
+        return channel.as_json()
+
     def getEpgData(self, sid, epgDate, days=2):
         """Get EPG data for the specified channel/date."""
         channelNode = self._getChannelNode(sid)
@@ -194,7 +191,7 @@ class SkyQRemote:
 
         epg = str(sid) + epgDate.strftime("%Y%m%d")
         if self._lastEpg == epg:
-            return self.channel
+            return self._channel
 
         programmes = set()
         for n in range(days):
@@ -211,20 +208,20 @@ class SkyQRemote:
         if len(programmes) == 0:
             _LOGGER.info(f"I0020 - Programme data not found for {sid} : {epgDate}")
 
-        self.channel = Channel(
+        self._channel = Channel(
             sid, channelno, channelname, channelImageUrl, sorted(programmes)
         )
 
-        return self.channel
+        return self._channel
 
     def getProgrammeFromEpgJSON(self, sid, epgDate, queryDate):
         """Get programme from EPG for specfied time and channel as json."""
         programme = self.getProgrammeFromEpg(sid, epgDate, queryDate)
-        if programme:
-            return programme.as_json()
-        else:
+        if not programme:
             return None
 
+        return programme.as_json()
+
     def getProgrammeFromEpg(self, sid, epgDate, queryDate):
         """Get programme from EPG for specfied time and channel."""
         epgData = self.getEpgData(sid, epgDate)
@@ -245,11 +242,11 @@ class SkyQRemote:
     def getCurrentLiveTVProgrammeJSON(self, sid):
         """Get current live programme on the specified channel as json."""
         programme = self.getCurrentLiveTVProgramme(sid)
-        if programme:
-            return programme.as_json()
-        else:
+        if not programme:
             return None
 
+        return programme.as_json()
+
     def getCurrentLiveTVProgramme(self, sid):
         """Get current live programme on the specified channel."""
         try:
@@ -260,17 +257,17 @@ class SkyQRemote:
 
             return programme
         except Exception as err:
-            _LOGGER.exception(f"X0030 - Error occurred: {self._host} : {sid} : {err}")
+            _LOGGER.exception(f"X0030 - Error occurred: {self.host} : {sid} : {err}")
             return None
 
     def getRecordingJSON(self, pvrId):
         """Get the recording details as json."""
         recording = self.getRecording(pvrId)
-        if recording:
-            return recording.as_json()
-        else:
+        if not recording:
             return None
 
+        return recording.as_json()
+
     def getRecording(self, pvrId):
         """Get the recording details."""
         season = None
@@ -320,19 +317,19 @@ class SkyQRemote:
         if isinstance(sequence, list):
             for item in sequence:
                 if item.casefold() not in self.commands:
-                    _LOGGER.error(f"E0010 - Invalid command: {self._host} : {item}")
+                    _LOGGER.error(f"E0010 - Invalid command: {self.host} : {item}")
                     break
                 self._sendCommand(self.commands[item.casefold()])
                 time.sleep(0.5)
         else:
             if sequence not in self.commands:
-                _LOGGER.error(f"E0020 - Invalid command: {self._host} : {sequence}")
+                _LOGGER.error(f"E0020 - Invalid command: {self.host} : {sequence}")
             else:
                 self._sendCommand(self.commands[sequence.casefold()])
 
     def _http_json(self, path, headers=None) -> str:
         response = requests.get(
-            REST_BASE_URL.format(self._host, self._jsonport, path),
+            REST_BASE_URL.format(self.host, self._jsonport, path),
             timeout=TIMEOUT,
             headers=headers,
         )
@@ -341,7 +338,7 @@ class SkyQRemote:
     def _getSoapControlURL(self, descriptionIndex):
         try:
             descriptionUrl = SOAP_DESCRIPTION_BASE_URL.format(
-                self._host, descriptionIndex
+                self.host, descriptionIndex
             )
             headers = {"User-Agent": SOAP_USER_AGENT}
             resp = requests.get(descriptionUrl, headers=headers, timeout=TIMEOUT)
@@ -361,21 +358,21 @@ class SkyQRemote:
                     return {"url": None, "status": "Not Found"}
                 return {
                     "url": SOAP_CONTROL_BASE_URL.format(
-                        self._host, playService["controlURL"]
+                        self.host, playService["controlURL"]
                     ),
                     "status": "OK",
                 }
             return {"url": None, "status": "Not Found"}
         except (requests.exceptions.Timeout):
             _LOGGER.warning(
-                f"W0020 - Control URL not accessible: {self._host} : {descriptionUrl}"
+                f"W0010 - Control URL not accessible: {self.host} : {descriptionUrl}"
             )
             return {"url": None, "status": "Error"}
         except (requests.exceptions.ConnectionError) as err:
-            _LOGGER.exception(f"X0070 - Connection error: {self._host} : {err}")
+            _LOGGER.exception(f"X0070 - Connection error: {self.host} : {err}")
             return {"url": None, "status": "Error"}
         except Exception as err:
-            _LOGGER.exception(f"X0010 - Other error occurred: {self._host} : {err}")
+            _LOGGER.exception(f"X0010 - Other error occurred: {self.host} : {err}")
             return {"url": None, "status": "Error"}
 
     def _callSkySOAPService(self, method):
@@ -403,25 +400,17 @@ class SkyQRemote:
 
     def _callSkyWebSocket(self, method):
         try:
-            client = _SkyWebSocket(WS_BASE_URL.format(self._host, method))
-            client.connect()
-            timeout = datetime.now() + timedelta(0, 5)
-            while client.data is None and datetime.now() < timeout:
-                pass
-            client.close()
-            if client.data is not None:
-                return json.loads(
-                    client.data.decode(DEFAULT_ENCODING), encoding=DEFAULT_ENCODING
-                )
-            return None
-        except (AttributeError) as err:
-            _LOGGER.debug(f"D0010 - Attribute Error occurred: {self._host} : {err}")
-            return None
-        except (TimeoutError):
-            _LOGGER.warning(f"W0030 - Websocket call failed: {self._host} : {method}")
+            ws = websocket.create_connection(WS_BASE_URL.format(self.host, method))
+            response = json.loads(ws.recv())
+            ws.close()
+            return response
+        except (TimeoutError) as err:
+            _LOGGER.warning(
+                f"W0020 - Websocket call failed: {self.host} : {method} : {err}"
+            )
             return {"url": None, "status": "Error"}
         except Exception as err:
-            _LOGGER.exception(f"X0020 - Error occurred: {self._host} : {err}")
+            _LOGGER.exception(f"X0020 - Error occurred: {self.host} : {err}")
             return None
 
     def _sendCommand(self, code):
@@ -433,15 +422,15 @@ class SkyQRemote:
             client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         except socket.error as err:
             _LOGGER.exception(
-                f"X0040 - Failed to create socket when sending command: {self._host} : {err}"
+                f"X0040 - Failed to create socket when sending command: {self.host} : {err}"
             )
             return
 
         try:
-            client.connect((self._host, self._port))
+            client.connect((self.host, self._port))
         except Exception as err:
             _LOGGER.exception(
-                f"X0050 - Failed to connect to client when sending command: {self._host} : {err}"
+                f"X0050 - Failed to connect to client when sending command: {self.host} : {err}"
             )
             return
 
@@ -464,7 +453,7 @@ class SkyQRemote:
 
             if time.time() > timeout:
                 _LOGGER.error(
-                    "E0030 - Timeout error sending command: {self._host} : {0}".format(
+                    "E0030 - Timeout error sending command: {self.host} : {0}".format(
                         str(code)
                     )
                 )
@@ -483,33 +472,11 @@ class SkyQRemote:
         channelno = channelNode["c"]
         return {"channel": channel, "channelno": channelno}
 
-    def _getDeviceInformation(self):
-        try:
-            resp = self._http_json(REST_PATH_DEVICEINFO)
-            return resp
-        except requests.exceptions.ConnectTimeout:
-            return None
-        except Exception as err:
-            _LOGGER.exception(f"X0080 - Error occurred: {self._host} : {err}")
-            return None
-
     def _setupDevice(self):
         """Set the remote up."""
         deviceInfo = self._getDeviceInformation()
         if not deviceInfo:
-            return None
-
-        alpha3 = None
-        if self._overrideCountry:
-            alpha3 = self._overrideCountry
-        elif "countryCode" in deviceInfo:
-            alpha3 = deviceInfo["countryCode"].upper()
-
-        if not alpha3:
-            _LOGGER.error(f"E0050 - No country identified: {self._host}")
-            return None
-
-        country = pycountry.countries.get(alpha_3=alpha3).alpha_2.casefold()
+            return
 
         url_index = 0
         self._soapControlURL = None
@@ -517,29 +484,59 @@ class SkyQRemote:
             self._soapControlURL = self._getSoapControlURL(url_index)["url"]
             url_index += 1
 
+        SkyQCountry = self._importCountry(deviceInfo)
+        if "serialNumber" in deviceInfo:
+            self.serialNumber = deviceInfo["serialNumber"]
+
+        self._remoteCountry = SkyQCountry(self.host)
+        self.country = self._remoteCountry.country
+
+        if not self._test_channel:
+            self._channels = self._http_json(REST_CHANNEL_LIST)
+        else:
+            self._channels = TEST_CHANNEL_LIST
+
+        self.deviceSetup = True
+        return
+
+    def _getDeviceInformation(self):
         try:
+            resp = self._http_json(REST_PATH_DEVICEINFO)
+            return resp
+        except (
+            requests.exceptions.ConnectTimeout,
+            requests.exceptions.ConnectionError,
+        ):
+            return None
+        except Exception as err:
+            _LOGGER.exception(f"X0080 - Error occurred: {self.host} : {err}")
+            return None
+
+    def _importCountry(self, deviceInfo):
+        alpha3 = None
+        if self._overrideCountry:
+            alpha3 = self._overrideCountry
+        elif "countryCode" in deviceInfo:
+            alpha3 = deviceInfo["countryCode"].upper()
+
+        if not alpha3:
+            _LOGGER.error(f"E0050 - No country identified: {self.host}")
+            return None
+
+        if alpha3 in KNOWN_COUNTRIES:
+            alpha3 = KNOWN_COUNTRIES[alpha3]
+
+        try:
+            country = pycountry.countries.get(alpha_3=alpha3).alpha_2.casefold()
             SkyQCountry = importlib.import_module(
                 "pyskyqremote.country.remote_" + country
             ).SkyQCountry
 
-            self._remoteCountry = SkyQCountry(self._host)
-            if not self._test_channel:
-                self._channels = self._http_json(REST_CHANNEL_LIST)
-            else:
-                self._channels = TEST_CHANNEL_LIST
+        except (AttributeError, ModuleNotFoundError) as err:
+            _LOGGER.warning(
+                f"W0030 - Invalid country, defaulting to GBR : {self.host} : {alpha3} : {err}"
+            )
 
-            self.deviceSetup = True
-            return "ok"
+            from pyskyqremote.country.remote_gb import SkyQCountry
 
-        except Exception as err:
-            _LOGGER.error(f"E0040 - Invalid country: {self._host} : {alpha3} : {err}")
-            return None
-
-
-class _SkyWebSocket(WebSocketClient):
-    def __init__(self, url):
-        super(_SkyWebSocket, self).__init__(url)
-        self.data = None
-
-    def received_message(self, message):
-        self.data = message.data
+        return SkyQCountry
